package com.flyfish.framework.beans.meta;

import com.flyfish.framework.controller.BaseController;
import com.flyfish.framework.controller.SafeController;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.repository.DefaultRepository;
import com.flyfish.framework.service.BaseService;
import com.flyfish.framework.service.impl.BaseServiceImpl;
import org.springframework.core.annotation.AliasFor;
import org.springframework.data.mongodb.core.mapping.Document;

import java.lang.annotation.*;

/**
 * RestBean扫描注解
 *
 * @author wangyu
 */
@Target({ElementType.TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Document
public @interface RestBean {

    /**
     * 生成的REST资源名称
     *
     * @return 资源名称
     */
    @AliasFor(value = "collection", annotation = Document.class)
    String value() default "";

    /**
     * 名称，可用于占位
     *
     * @return 结果
     */
    String name() default "";

    /**
     * 排除的属性
     * @return 结果
     */
    String[] exclude() default {};

    /**
     * 必须指定qo
     *
     * @return 结果
     */
    Class<? extends Qo> queryClass() default NameLikeQo.class;

    /**
     * 仓库的超类，默认是default
     *
     * @return 结果
     */
    Class<? extends DefaultRepository> repoClass() default DefaultRepository.class;

    /**
     * 服务类，动态生成服务
     *
     * @return 结果
     */
    Class<? extends BaseService> serviceClass() default BaseServiceImpl.class;

    /**
     * controller的类，支持自定义
     *
     * @return 结果
     */
    Class<? extends SafeController> controllerClass() default BaseController.class;
}
