package com.flyfish.framework.service;

import com.flyfish.framework.domain.po.User;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.userdetails.ReactiveUserDetailsPasswordService;
import org.springframework.security.core.userdetails.ReactiveUserDetailsService;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

/**
 * spring security用户详情服务
 *
 * @author wangyu
 */
@Qualifier("mongoUserDetailsService")
public interface MongoUserDetailsService extends ReactiveUserDetailsService, ReactiveUserDetailsPasswordService {

    /**
     * 针对微信端，即时校验登录
     *
     * @param user     用户
     * @param exchange 数据
     * @return 结果
     */
    Mono<Authentication> authenticate(UserDetails user, ServerWebExchange exchange);

    /**
     * 加载上下文，提供token调用
     *
     * @param user 用户信息
     * @return 结果
     */
    Mono<SecurityContext> loadContext(UserDetails user);

    /**
     * 主动退出登录
     *
     * @param exchange 数据
     * @return 结果
     */
    Mono<Void> logout(ServerWebExchange exchange);

    /**
     * 发生错误时的处理
     *
     * @param exchange 交换
     * @return 结果
     */
    Mono<User> error(ServerWebExchange exchange);
}
